#version 430 core

uniform sampler2D textureMap;
//uniform sampler2D normalMap;

uniform vec3 lightColor;
uniform float ambientCoeff;
uniform float diffuseCoeff;
uniform float attenuationConst;
uniform float attenuationLinear;
uniform float attenuationExp;

uniform float specularCoeff;
uniform float specularPower;

in vec2 fragUV;
in vec3 fragNormal;

in vec3 fragCamViewDir;
in vec3 fragCamLightDir;

out vec4 color;

vec4 blinnPhongShading(vec3 L, vec3 V, vec3 N)
{
	vec4 textureColor = texture(textureMap, fragUV);
	vec4 textureColorLight = textureColor * vec4(lightColor, 1.0);

	float camLightLength = length(fragCamLightDir);

	//ambient
	vec4 ambientColor = textureColorLight * ambientCoeff;

	// diffuse
	float diffuseIntensity = clamp(dot(N,L), 0, 1);
	vec4 diffuseColor = textureColorLight * diffuseIntensity * diffuseCoeff;

	// specular
	vec3 H = normalize(L + V);
	float specularIntensity = clamp(dot(N, H), 0, 1);
	specularIntensity = pow(specularIntensity, specularPower);
	vec4 specularColor = vec4(lightColor, 1.0) * specularCoeff * specularIntensity;

	// attenuation
	float attenuation = attenuationConst + attenuationLinear * camLightLength + attenuationExp * camLightLength * camLightLength;

	return vec4((ambientColor + diffuseColor + specularColor).xyz, 1.0) / attenuation;
}

void main(void)
{
	color = blinnPhongShading(normalize(fragCamLightDir), normalize(fragCamViewDir), normalize(fragNormal));
}
