#version 430 core

uniform mat4 mvpMatrix;
uniform mat4 mvMatrix;
uniform mat4 modelMatrix;
uniform mat4 viewMatrix;
uniform mat3 normalMatrix;

uniform vec3 lightPosition;

layout(location=0) in vec3 position;
layout(location=1) in vec2 uv;
layout(location=2) in vec3 normal;
layout(location=3) in vec3 tangent;
layout(location=4) in vec3 bitangent;

out vec2 fragUV;

out vec3 fragTangentViewDir;
out vec3 fragTangentLightDir;
out float fragLightVPosLength;

void main(void)
{
	gl_Position = mvpMatrix * vec4(position, 1.0);
	fragUV = uv;

	vec3 camVPos = (mvMatrix * vec4(position, 1.0)).xyz;
	vec3 camViewDir = vec3(0.0, 0.0, 0.0) - camVPos;
	vec3 camLightPos = (viewMatrix * vec4(lightPosition, 1.0)).xyz;
	vec3 camLightDir = camLightPos - camVPos;
	fragLightVPosLength = length(camLightDir);

	mat3 mvMatrix3x3 = mat3(mvMatrix);

	vec3 camTangent = mvMatrix3x3 * tangent;
	vec3 camBitangent = mvMatrix3x3 * bitangent;
	vec3 camNormal = mvMatrix3x3 * normal;

	mat3 tbnMatrix = transpose(mat3(
		camTangent,
		camBitangent,
		camNormal
	));
		
	fragTangentViewDir = tbnMatrix * camViewDir;
	fragTangentLightDir = tbnMatrix * camLightDir;
}

