#version 430 core

uniform sampler2D textureMap;
uniform sampler2D normalMap;

uniform vec3 lightColor;
uniform float ambientCoeff;
uniform float diffuseCoeff;

uniform float specularCoeff;
uniform float specularPower;

in vec2 fragUV;

in vec3 fragTangentViewDir;
in vec3 fragTangentLightDir;
in float fragLightVPosLength;

out vec4 color;

vec4 blinnPhongShading(vec3 L, vec3 V, vec3 N)
{
	vec4 textureColor = texture(textureMap, fragUV);
	vec4 textureColorLight = textureColor * vec4(lightColor, 1.0);

	//ambient
	vec4 ambientColor = textureColorLight * ambientCoeff;

	// diffuse
	float diffuseIntensity = clamp(dot(N,L), 0, 1);
	vec4 diffuseColor = textureColorLight * diffuseIntensity * diffuseCoeff;

	// specular
	vec3 H = normalize(L + V);
	float specularIntensity = clamp(dot(N, H), 0, 1);
	specularIntensity = pow(specularIntensity, specularPower);
	vec4 specularColor = vec4(lightColor, 1.0) * specularCoeff * specularIntensity;

	return vec4((ambientColor + diffuseColor + specularColor).xyz, 1.0);
}

void main(void)
{
	color = blinnPhongShading(normalize(fragTangentLightDir), normalize(fragTangentViewDir), normalize((texture2D(normalMap, fragUV).rgb)*2.0 - 1.0));
}
