#version 430 core

const float SHADOW_BIAS = 0.15;
const float FAR_PLANE = 40.0;
const float SHADOW_FACTOR = 0.5;

uniform sampler2D textureMap;
uniform sampler2D normalMap;
uniform samplerCube cubeMap;

uniform vec3 lightPosition;
uniform vec3 lightColor;
uniform float ambientCoeff;
uniform float diffuseCoeff;
uniform float attenuationConst;
uniform float attenuationLinear;
uniform float attenuationExp;

uniform float specularCoeff;
uniform float specularPower;

in vec2 fragUV;
in vec4 fragVPos;

in vec3 fragTangentViewDir;
in vec3 fragTangentLightDir;
in float fragLightVPosLength;

out vec4 color;

float fragShadow()
{
	vec3 shadowMapLookup = fragVPos.xyz - lightPosition;
	float shadowValue = texture(cubeMap, shadowMapLookup).r;
	// maybe incorrect?
	if(shadowValue == 1.0f)
	{
		return 1.0f;
	}
	float shadowMapLength = shadowValue * FAR_PLANE;
	float fragLightLength = length(shadowMapLookup);
	if((shadowMapLength + SHADOW_BIAS) < fragLightLength)
	{
		return SHADOW_FACTOR;
	}
	else
	{
		return 1.0f;
	}
}

vec4 blinnPhongShading(vec3 L, vec3 V, vec3 N)
{
	vec4 textureColor = texture(textureMap, fragUV);
	vec4 textureColorLight = textureColor * vec4(lightColor, 1.0);

	//ambient
	vec4 ambientColor = textureColorLight * ambientCoeff;

	// diffuse
	float diffuseIntensity = clamp(dot(N,L), 0, 1);
	vec4 diffuseColor = textureColorLight * diffuseIntensity * diffuseCoeff;

	// specular
	vec3 H = normalize(L + V);
	float specularIntensity = clamp(dot(N, H), 0, 1);
	specularIntensity = pow(specularIntensity, specularPower);
	vec4 specularColor = vec4(lightColor, 1.0) * specularCoeff * specularIntensity;

	// attenuation
	float attenuation = attenuationConst + attenuationLinear * fragLightVPosLength + attenuationExp * fragLightVPosLength * fragLightVPosLength;

	return vec4((ambientColor + fragShadow() * diffuseColor + fragShadow() * specularColor).xyz, 1.0) / attenuation;
}

void main(void)
{
	color = blinnPhongShading(normalize(fragTangentLightDir), normalize(fragTangentViewDir), normalize(texture2D(normalMap, fragUV).rgb*2.0 - 1.0));
}